import { UmzugStorage } from './storage';
import { MergeExclusive } from './type-util';
import * as emittery from 'emittery';
export declare type Promisable<T> = T | PromiseLike<T>;
export declare type LogFn = (message: Record<string, unknown>) => void;
/** Constructor options for the Umzug class */
export interface UmzugOptions<Ctx = never> {
    /** The migrations that the Umzug instance should perform */
    migrations: InputMigrations<Ctx>;
    /** A logging function. Pass `console` to use stdout, or pass in your own logger. Pass `undefined` explicitly to disable logging. */
    logger: Record<'info' | 'warn' | 'error' | 'debug', LogFn> | undefined;
    /** The storage implementation. By default, `JSONStorage` will be used */
    storage?: UmzugStorage;
    /** An optional context object, which will be passed to each migration function, if defined */
    context?: Ctx;
    /** Options for file creation */
    create?: {
        /**
         * A function for generating placeholder migration files. Specify to make sure files generated using `.create` follow team conventions.
         * Should return an array of [filepath, content] pairs. Usually, only one pair is needed, but to put `down` migrations in a separate
         * file, more than one can be returned.
         */
        template?: (filepath: string) => Array<[string, string]>;
        /**
         * The default folder that new migration files should be generated in. If this is not specified, the new migration file will be created
         * in the same folder as the last existing migration. The value here can be overriden by passing `folder` when calling `create`.
         */
        folder?: string;
    };
}
/** Serializeable metadata for a migration. The structure returned by the external-facing `pending()` and `executed()` methods. */
export interface MigrationMeta {
    /** Name - this is used as the migration unique identifier within storage */
    name: string;
    /** An optional filepath for the migration. Note: this may be undefined, since not all migrations correspond to files on the filesystem */
    path?: string;
}
export interface MigrationParams<T> {
    name: string;
    path?: string;
    context: T;
}
/**
 * A runnable migration. Represents a migration object with an `up` function which can be called directly, with no arguments, and an optional `down` function to revert it.
 */
export interface RunnableMigration<T> extends MigrationMeta {
    /** The effect of applying the migration */
    up: (params: MigrationParams<T>) => Promise<unknown>;
    /** The effect of reverting the migration */
    down?: (params: MigrationParams<T>) => Promise<unknown>;
}
/** Glob instructions for migration files */
export declare type GlobInputMigrations<T> = {
    /**
     * A glob string for migration files. Can also be in the format `[path/to/migrations/*.js', {cwd: 'some/base/dir', ignore: '**ignoreme.js' }]`
     * See https://npmjs.com/package/glob for more details on the glob format - this package is used internally.
     */
    glob: string | [string, {
        cwd?: string;
        ignore?: string | string[];
    }];
    /** Will be supplied to every migration function. Can be a database client, for example */
    /** A function which takes a migration name, path and context, and returns an object with `up` and `down` functions. */
    resolve?: Resolver<T>;
};
/**
 * Allowable inputs for migrations. Can be either glob instructions for migration files, a list of runnable migrations, or a
 * function which receives a context and returns a list of migrations.
 */
export declare type InputMigrations<T> = GlobInputMigrations<T> | Array<RunnableMigration<T>> | ((context: T) => Promisable<Array<RunnableMigration<T>>>);
/** A function which takes a migration name, path and context, and returns an object with `up` and `down` functions. */
export declare type Resolver<T> = (params: MigrationParams<T>) => RunnableMigration<T>;
export declare const RerunBehavior: {
    /** Hard error if an up migration that has already been run, or a down migration that hasn't, is encountered */
    readonly THROW: "THROW";
    /** Silently skip up migrations that have already been run, or down migrations that haven't */
    readonly SKIP: "SKIP";
    /** Re-run up migrations that have already been run, or down migrations that haven't */
    readonly ALLOW: "ALLOW";
};
export declare type RerunBehavior = keyof typeof RerunBehavior;
export declare type MigrateUpOptions = MergeExclusive<{
    /** If specified, migrations up to and including this name will be run. Otherwise, all pending migrations will be run */
    to?: string;
}, {
    /** Only run this many migrations. If not specified, all pending migrations will be run */
    step: number;
}, {
    /** If specified, only the migrations with these names migrations will be run. An error will be thrown if any of the names are not found in the list of available migrations */
    migrations: string[];
    /** What to do if a migration that has already been run is explicitly specified. Default is `THROW`. */
    rerun?: RerunBehavior;
}>;
export declare type MigrateDownOptions = MergeExclusive<{
    /** If specified, migrations down to and including this name will be revert. Otherwise, only the last executed will be reverted */
    to?: string | 0;
}, {
    /** Revert this many migrations. If not specified, only the most recent migration will be reverted */
    step: number;
}, {
    /**
     * If specified, only the migrations with these names migrations will be reverted. An error will be thrown if any of the names are not found in the list of executed migrations.
     * Note, migrations will be run in the order specified.
     */
    migrations: string[];
    /** What to do if a migration that has not been run is explicitly specified. Default is `THROW`. */
    rerun?: RerunBehavior;
}>;
export declare class Umzug<Ctx> extends emittery.Typed<Record<'migrating' | 'migrated' | 'reverting' | 'reverted', MigrationParams<Ctx>> & Record<'beforeAll' | 'afterAll', {
    context: Ctx;
}>> {
    private readonly storage;
    /**
     * Compile-time only property for type inference. After creating an Umzug instance, it can be used as type alias for
     * a user-defined migration. The function receives a migration name, path and the context for an umzug instance
     * @example
     * // migrator.ts
     * import { Umzug } from 'umzug'
     *
     * const umzug = new Umzug({...})
     * export type Migration = typeof umzug._types.migration;
     *
     * umzug.up();
     *
     * @example
     * // migration-1.ts
     * import type { Migration } from '../migrator'
     *
     * // name and context will now be strongly-typed
     * export const up: Migration = ({name, context}) => context.query(...)
     * export const down: Migration = ({name, context}) => context.query(...)
     */
    readonly _types: {
        migration: (params: MigrationParams<Ctx>) => Promise<unknown>;
    };
    /** creates a new Umzug instance */
    constructor(
    /** @internal */
    options: UmzugOptions<Ctx>);
    private get context();
    private logging;
    static defaultResolver: Resolver<unknown>;
    /**
     * 'Run' an umzug instance as a CLI. This will read `process.argv`, execute commands based on that, and call
     * `process.exit` after running. If that isn't what you want, stick to the programmatic API.
     * You probably want to run only if a file is executed as the process's 'main' module with something like:
     * @example
     * if (require.main === module) {
     *   myUmzugInstance.runAsCLI()
     * }
     */
    runAsCLI(argv?: string[]): Promise<boolean>;
    /**
     * create a clone of the current Umzug instance, allowing customising the list of migrations.
     * This could be used, for example, to sort the list of migrations in a specific order.
     */
    extend(transform: (migrations: ReadonlyArray<RunnableMigration<Ctx>>) => Promisable<Array<RunnableMigration<Ctx>>>): Umzug<Ctx>;
    /** Get the list of migrations which have already been applied */
    executed(): Promise<MigrationMeta[]>;
    /** Get the list of migrations which have already been applied */
    private _executed;
    /** Get the list of migrations which are yet to be applied */
    pending(): Promise<MigrationMeta[]>;
    private _pending;
    private withBeforeAfterHooks;
    /**
     * Apply migrations. By default, runs all pending migrations.
     * @see MigrateUpOptions for other use cases using `to`, `migrations` and `rerun`.
     */
    up(options?: MigrateUpOptions): Promise<MigrationMeta[]>;
    /**
     * Revert migrations. By default, the last executed migration is reverted.
     * @see MigrateDownOptions for other use cases using `to`, `migrations` and `rerun`.
     */
    down(options?: MigrateDownOptions): Promise<MigrationMeta[]>;
    create(options: {
        name: string;
        folder?: string;
        prefix?: 'TIMESTAMP' | 'DATE' | 'NONE';
        allowExtension?: string;
        allowConfusingOrdering?: boolean;
        skipVerify?: boolean;
    }): Promise<void>;
    private static defaultCreationTemplate;
    private findNameIndex;
    private findMigrations;
    /** helper for parsing input migrations into a callback returning a list of ready-to-run migrations */
    private getMigrationsResolver;
}
